#ifndef LIBCROCO_VERSION_NUMBER
#define LIBCROCO_VERSION_NUMBER 612
#endif

#ifndef LIBCROCO_VERSION
#define LIBCROCO_VERSION "0.6.12"
#endif

#ifndef G_DISABLE_CHECKS
#if 0
#define G_DISABLE_CHECKS 0
#endif
#endif

#ifndef CROCO_HAVE_LIBXML2
#define CROCO_HAVE_LIBXML2 (1)
#endif
